import photoshop as ps

# Get photoshop instance.
app = ps.Application()
# Get current active document.
doc = app.activeDocument

# Add a solid color.
textColor = ps.SolidColor()
textColor.rgb.red = 43.0
textColor.rgb.green = 197.0
textColor.rgb.blue = 0.0

# Create empty layer.
new_text_layer = doc.artLayers.add()
# Set empty layer type to text layer
new_text_layer.kind = ps.LayerKind.TextLayer
# Set current text layer contents to "Hello, World!".
new_text_layer.textItem.contents = 'Hello, World!'
# Change current text layer position.
new_text_layer.textItem.position = [160, 167]
# Change current text layer text size.
new_text_layer.textItem.size = 36
# Change current text layer color.
new_text_layer.textItem.color = textColor



# Import local modules
from photoshop._core import Photoshop
from photoshop.artlayer import ArtLayer
from photoshop.artlayers import ArtLayers
from photoshop.errors import COMError
from photoshop.layers import Layers
from photoshop.layerSets import LayerSets
from photoshop.selection import Selection
from photoshop.layerSet import LayerSet
from photoshop.enumerations import ExtensionType
from photoshop.enumerations import SaveOptions


class Document(Photoshop):
    object_name = 'Application'

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def artLayers(self):
        return ArtLayers(self.app.artLayers)

    @property
    def activeLayer(self):
        type_ = self.eval_javascript("app.activeDocument.activeLayer.typename")
        mappings = {"LayerSet": LayerSet,
                    "ArtLayer": ArtLayer}
        func = mappings[type_]
        return func(self.app.activeLayer)

    @activeLayer.setter
    def activeLayer(self, item):
        self.app.activeLayer = item

    @property
    def activeChannels(self):
        """The selected channels."""
        return self.app.activeChannels

    @activeChannels.setter
    def activeChannels(self, channels):
        self.app.activeChannels = channels

    @property
    def activeHistoryBrushSource(self):
        """The history state to use with the history brush."""
        return self.app.activeHistoryBrushSource

    @property
    def activeHistoryState(self):
        """The current history state for this document."""
        return self.app.activeHistoryState

    @property
    def backgroundLayer(self):
        """The background layer for the Document."""
        return self.app.backgroundLayer

    @property
    def bitsPerChannel(self):
        """The number of bits per channel."""
        return self.app.bitsPerChannel

    @property
    def channels(self):
        return self.app.channels

    @property
    def colorProfileName(self):
        """The name of the color profile. Valid only when no value is specified
        for color profile kind (to indicate a custom color profile)."""
        return self.app.colorProfileName

    @property
    def colorProfileType(self):
        """The type of color model that defines the working space of the
        Document."""
        return self.app.colorProfileType

    @property
    def colorSamplers(self):
        """The current color samplers associated with the Document."""
        return self.app.colorSamplers

    @property
    def componentChannels(self):
        """The color component channels for this Document."""
        return self.app.componentChannels

    @property
    def countItems(self):
        """The current count items in the Document."""
        return self.app.countItems

    @property
    def fullName(self):
        """The full path name of the Document."""
        try:
            return self.app.fullName
        except COMError:
            self.eval_javascript(
                'alert ("Please save your Document first!",'
                '"{}")'.format(self.title),
            )

    @property
    def height(self):
        """The height of the Document."""
        return self.app.Height

    @property
    def histogram(self):
        """A histogram showing the number of pixels at each color intensity
        level for the composite channel."""
        return self.app.Histogram

    @property
    def history_states(self):
        """The history states collection in this Document."""
        return self.app.HistoryStates

    @property
    def id(self):
        """The unique ID of this Document."""
        return self.app.Id

    @property
    def info(self):
        """Metadata about the Document."""
        return self.app.info

    @property
    def layerComps(self):
        """The layer comps collection in this Document."""
        return self.app.LayerComps

    @property
    def layers(self):
        """The layers collection in the Document."""
        return Layers(self.app.Layers)

    @property
    def layerSets(self):
        """The layer sets collection in the Document."""
        return LayerSets(self.app.layerSets)

    @property
    def managed(self):
        """If true, the Document is a workgroup Document."""
        return self.app.Managed

    @property
    def measurement_scale(self):
        """The measurement scale of the Document."""
        return self.app.MeasurementScale

    @property
    def mode(self):
        """The color profile."""
        return self.app.Mode

    @property
    def name(self):
        """The Document name."""
        return self.app.name

    @property
    def parent(self):
        """The object's container."""
        return self.app.Parent

    @property
    def path(self):
        """The path to the Document."""
        try:
            return self.app.Path
        except COMError:
            self.eval_javascript(
                'alert ("Please save your Document first!",'
                '"{}")'.format(self.title),
            )

    @path.setter
    def path(self, path):
        self.app.fullName = path

    @property
    def pathItems(self):
        return self.app.pathItems

    @property
    def pixelAspectRatio(self):
        """The (custom) pixel aspect ratio of the Document.

        Range: 0.100 to 10.000.

        """
        return self.app.pixelAspectRatio

    @property
    def printSettings(self):
        """Document print settings."""
        return self.app.printSettings

    @property
    def quickMaskMode(self):
        """If true, the document is in Quick Mask mode."""
        return self.app.quickMaskMode

    @property
    def saved(self):
        """If true, the Document been saved since the last change."""
        return self.app.Saved

    @property
    def resolution(self):
        """The resolution of the Document (in pixels per inch)"""
        return self.app.resolution

    @property
    def selection(self):
        """The selected area of the Document."""
        return Selection(self.app.selection)

    @property
    def typename(self):
        """The class name of the object."""
        return self.app.typename

    @property
    def width(self):
        return self.app.Width

    @property
    def xmpMetadata(self):
        """The XMP properties of the Document. The Camera RAW settings are
        stored here."""
        return self.app.xmpMetadata

    # Methods
    def autoCount(self, *args, **kwargs):
        """Counts the objects in the Document."""
        return self.app.autoCount(*args, **kwargs)

    def changeMode(self, *args, **kwargs):
        """Changes the mode of the Document."""
        return self.app.changeMode(*args, **kwargs)

    def close(self, saving=SaveOptions.DoNotSaveChanges):
        return self.app.close(saving)

    def convertProfile(self):
        return self.app.convertProfile()

    def flatten(self):
        """Flattens all layers."""
        return self.app.Flatten()

    def mergeVisibleLayers(self):
        """Flattens all visible layers in the Document."""
        return self.app.mergeVisibleLayers()

    def crop(self, **kwargs):
        return self.app.Crop(**kwargs)

    def exportDocument(self, *args, **kwargs):
        """Exports the Document."""
        return self.app.exportDocument(*args, **kwargs)

    def duplicate(self, name=None, merge_layers_only=False):
        return Document(self.app.duplicate(name, merge_layers_only))

    def paste(self):
        """Pastes contents of the clipboard into the Document."""
        return self.app.paste()

    def print(self):
        """Prints the document."""
        return self.app.print()

    def printOneCopy(self):
        self.app.printOneCopy()

    def rasterizeAllLayers(self):
        return self.app.rasterizeAllLayers()

    def recordMeasurements(self, source, dataPoints):
        """Records the measurements of document."""
        self.app.recordMeasurements(source, dataPoints)

    def reveal_all(self):
        """Expands the Document to show clipped sections."""
        return self.app.revealAll()

    def save(self):
        """Saves the Document."""
        return self.app.save()

    def saveAs(self, file_path, options, asCopy=True,
               extensionType=ExtensionType.Lowercase):
        """Saves the documents with the specified save options.

        Args:
            file_path (str): Absolute path of psd file.
            options (photoshop.JPEGSaveOptions): Save options.
            asCopy (bool):
        """
        return self.app.saveAs(
            file_path, options, asCopy,
            extensionType,
        )

    def splitChannels(self):
        """Splits the channels of the document."""
        self.app.splitChannels()

    def suspendHistory(self, historyString, javaScriptString):
        """Provides a single history state for the entire script.

        Allows a single undo for all actions taken in the script.

        """
        self.eval_javascript(
            f"app.activeDocument.suspendHistory('{historyString}',"
            f" '{javaScriptString}')")

    def trap(self, width):
        """Applies trapping to a CMYK document. Valid only when ‘mode’ = CMYK."""
        self.app.trap(width)

    def trim(self, *args, **kwargs):
        return self.app.trim(*args, **kwargs)

    def resizeImage(self, width, height, resolution=72, psAutomatic=8):
        """Changes the size of the image.

        Args:
            width: The desired width of the image.
            height: The desired height of the image.
            resolution: The resolution (in pixels per inch)

        Returns:

        """
        return self.app.resizeImage(
            width, height, resolution,
            psAutomatic,
        )



from photoshop._core import Photoshop
from photoshop.enumerations import AnchorPosition
from photoshop.enumerations import LayerKind


class Layer(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def kind(self):
        return LayerKind(self.adobe.activeDocument.activeLayer.kind)

    @kind.setter
    def kind(self, value):
        self.adobe.activeDocument.activeLayer.kind = value
        # Convert current layer to ArtLayer.
        self.app = self.adobe.activeDocument.activeLayer

    @property
    def allLocked(self):
        return self.app.allLocked

    @property
    def blendMode(self):
        return self.app.blendMode

    @property
    def bounds(self):
        return self.app.bounds

    @property
    def boundsNoEffects(self):
        return self.app.boundsNoEffects

    @property
    def id(self):
        return self.app.id

    @property
    def itemIndex(self):
        return self.app.itemIndex

    @property
    def linkedLayers(self):
        return self.app.linkedLayers

    @property
    def name(self):
        return self.app.name

    @name.setter
    def name(self, text):
        self.app.name = text

    @property
    def opacity(self):
        return self.app.opacity

    @property
    def parent(self):
        return self.app.parent

    @property
    def typename(self):
        return self.eval_javascript("app.activeDocument.activeLayer.typename")

    @property
    def visible(self):
        return self.app.visible

    @property
    def xmpMetadata(self):
        return self.app.xmpMetadata

    @property
    def isBackgroundLayer(self):
        return self.app.isBackgroundLayer

    def duplicate(self):
        self.app.duplicate()

    def link(self, layer):
        self.app.link(layer)

    def move(self):
        self.app.move()

    def moveToEnd(self, layerSet):
        self.app.moveToEnd(layerSet)

    def remove(self):
        """Deletes this object."""
        self.app.remove()

    def removeAll(self):
        """Deletes all elements."""
        self.app.removeAll()

    def resize(self, horizontal, vertical, anchor=AnchorPosition.MiddleCenter):
        """Scales the object.

        Args:
            horizontal (int): The amount to scale the object horizontally
                (as a percentage).
            vertical (int): The amount to scale the object vertically
                (as a percentage).
            anchor (): The point to resize about.

        """
        self.app.resize(horizontal, vertical, anchor)

    def rotate(self, angle):
        self.app.rotate(angle)

    def translate(self, x, y):
        self.app.translate(x, y)

    def unlink(self):
        self.app.unlink()



presetKind = 'presetKindType'
smartSharpen = 'smartSharpen'
presetKindType = 'presetKindType'
presetKindCustom = 'presetKindCustom'
noiseReduction = 'noiseReduction'
blur = 'blur'
blurType = 'blurType'
AMNT = 'Amnt'
contentLayer = 'contentLayer'

# Name of photoshop ``AdobeScriptAutomation Scripts``.
ADOBE_SCRIPT_AUTOMATION_SCRIPTS = 'AdobeScriptAutomation Scripts'

# The photoshop version to COM progid mappings.
PHOTOSHOP_VERSION_MAPPINGS = {
    '2020': 140,
    '2019': 130,
    '2018': 120,
    '2017': 110,
}

# Values for Photoshop enumeration 'PsLayerKind'.
NORMAL_LAYER = 1
TEXT_LAYER = 2
SOLID_FILL_LAYER = 3

# Constants of Photoshop action.
# The all action constants from ``ScriptingListenerJS`` plugin.
# More ``StringIDs`` you can check out:
# http://www.tonton-pixel.com/wp-content/uploads/DecisionTable.pdf
NULL = 'null'
GSNB = 'GsnB'
ORDN = 'Ordn'
DOCI = 'DocI'
CLS = 'Cls '  # This string ID requires a space.
LYR = 'Lyr '  # This string ID requires a space.
OFST = 'Ofst'
PRC = '#Prc'
RDS = 'Rds '  # This string ID requires a space.
PX1 = '#Pxl'
PLC = 'Plc '  # This string ID requires a space.
IDNT = 'Idnt'
HRZN = 'Hrzn'
IN = 'In  '  # This string ID requires two spaces.
TRGT = 'Trgt'
SAVE = 'save'
SAVE_STAGE = 'saveStage'
SAVE_STAGE_TYPE = 'saveStageType'
SAVE_SUCCEEDED = 'saveSucceeded'
RASTERIZE_LAYER = 'rasterizeLayer'
YSN = 'YsN '  # This string ID requires a space.
N = 'N   '  # This string ID requires three spaces.
VRTC = 'Vrtc'
SVNG = 'Svng'
FTCS = 'FTcs'
QCST = 'QCSt'
QCSA = 'Qcsa'
FORCE_NOTIFY = 'forceNotify'
JSMS = 'jsMs'
JSCT = 'jsCt'
PLACED_LAYER_EDIT_CONTENTS = 'placedLayerEditContents'

# The photoshop save options.
JPEG_SAVE_OPTIONS = 'JPEGSaveOptions'
PNG_SAVE_OPTIONS = 'PNGSaveOptions'
PHOTOSHOP_SAVE_OPTIONS = 'PhotoshopSaveOptions'

# Controls whether Photoshop displays dialogs during scripts.
# Show all dialogs.
DIALOG_MODES_ALL = 0
# Show only dialogs related to errors.
DIALOG_MODES_ERROR = 1
# Show no dialogs.
DIALOG_MODES_NO = 2

# The measurement unit for ruler increments.
UNITS_CM = 6  # Centimeters.
UNITS_INCHES = 5  # Inches.
UNITS_MM = 4  # Millimeters.
UNITS_PERCENT = 3  # Percent.
UNITS_PICAS = 2  # Picas.
UNITS_PIXELS = 1  # Pixels.
UNITS_POINTS = 0  # Points.



from photoshop.save_options.gif import GIFSaveOptions
from photoshop.save_options.jpg import JPEGSaveOptions
from photoshop.save_options.pdf import PDFSaveOptions
from photoshop.save_options.png import ExportOptionsSaveForWeb
from photoshop.save_options.png import PNGSaveOptions
from photoshop.save_options.psd import PhotoshopSaveOptions
from photoshop.save_options.tif import TiffSaveOptions

# Do not save changes.
DONOTSAVECHANGES = 2
# Ask the user whether to save.
PROMPTTOSAVECHANGES = 1
# Save changes.
SAVECHANGES = 3

__all__ = [
    'GIFSaveOptions',
    'JPEGSaveOptions',
    'PDFSaveOptions',
    'ExportOptionsSaveForWeb',
    'PNGSaveOptions',
    'PhotoshopSaveOptions',
    'TiffSaveOptions',
]



# This script will demonstrate how to load a selection from a saved alpha
# channel.

from photoshop import Session

with Session() as ps:
    doc_ref = ps.app.documents.add(320, 240)
    start_ruler_units = ps.app.preferences.rulerUnits
    if start_ruler_units is not ps.Units.Pixels:
        ps.app.Preferences.RulerUnits = ps.Units.Pixels
    # Save a rectangular selection area offset by 50 pixels from the image
    # border into an alpha channel.
    offset = 50
    selBounds1 = ((offset, offset), (doc_ref.Width - offset, offset),
                  (doc_ref.Width - offset, doc_ref.Height - offset),
                  (offset, doc_ref.Height - offset))
    doc_ref.selection.select(selBounds1)
    selAlpha = doc_ref.channels.Add()
    doc_ref.selection.store(selAlpha)

    # Now create a second wider but less tall selection.
    selBounds2 = ((0, 75), (doc_ref.Width, 75), (doc_ref.Width, 150), (0, 150))
    doc_ref.selection.select(selBounds2)

    # Load the selection from the just saved alpha channel, combining it with
    # the active selection.
    doc_ref.selection.load(selAlpha, ps.SelectionType.ExtendSelection, False)

    # Set ruler back to where it was.
    ps.app.Preferences.RulerUnits = start_ruler_units



"""
References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/ActiveLayer.py

"""
import photoshop as ps

app = ps.Application()

if app.documents.length < 1:
    docRef = app.documents.add()
else:
    docRef = app.activeDocument

if docRef.layers.length < 2:
    docRef.artLayers.add()

activeLayerName = docRef.activeLayer.name
if docRef.activeLayer.name != docRef.layers.item(docRef.layers.length).name:
    docRef.activeLayer = docRef.layers.item(docRef.layers.length)
else:
    docRef.activeLayer = docRef.layers.item(1)



import photoshop as ps

app = ps.Application()

# psDisplayNoDialogs is a PS COM constant, see pscc2018.py or scripting COM
psDisplayNoDialogs = 3
for index, x in enumerate(range(50)):
    # Execute an existing action from action palette
    idPly = app.charIDToTypeID('Ply ')
    desc8 = ps.ActionDescriptor()
    idnull = app.charIDToTypeID('null')
    ref3 = ps.ActionReference()
    idActn = app.charIDToTypeID('Actn')
    ref3.putName(idActn, 'Sepia Toning (layer)')
    idASet = app.charIDToTypeID('ASet')
    ref3.PutName(idASet, 'Default Actions')
    desc8.putReference(idnull, ref3)
    app.executeAction(idPly, desc8, psDisplayNoDialogs)

    # Create solid color fill layer.
    idMk = app.charIDToTypeID('Mk  ')
    desc21 = ps.ActionDescriptor()
    idNull = app.charIDToTypeID('null')
    ref12 = ps.ActionReference()
    idContentLayer1 = app.stringIDToTypeID('contentLayer')
    ref12.putClass(idContentLayer1)
    desc21.putReference(idNull, ref12)
    idUsng = app.charIDToTypeID('Usng')
    desc22 = ps.ActionDescriptor()
    idType = app.charIDToTypeID('Type')
    desc23 = ps.ActionDescriptor()
    idClr = app.charIDToTypeID('Clr ')
    desc24 = ps.ActionDescriptor()
    idRd = app.charIDToTypeID('Rd  ')
    desc24.putDouble(idRd, index)
    idGrn = app.charIDToTypeID('Grn ')
    desc24.putDouble(idGrn, index)
    idBl = app.charIDToTypeID('Bl  ')
    desc24.putDouble(idBl, index)
    idRGBC = app.charIDToTypeID('RGBC')
    desc23.putObject(idClr, idRGBC, desc24)
    idSolidColorLayer = app.StringIDToTypeID('solidColorLayer')
    desc22.putObject(idType, idSolidColorLayer, desc23)
    idContentLayer2 = app.StringIDToTypeID('contentLayer')
    desc21.putObject(idUsng, idContentLayer2, desc22)
    app.executeAction(idMk, desc21, psDisplayNoDialogs)

    # Select mask
    idSlct = app.charIDToTypeID('slct')
    desc38 = ps.ActionDescriptor()
    idNull1 = app.charIDToTypeID('null')
    ref20 = ps.ActionReference()
    idChnl1 = app.charIDToTypeID('Chnl')
    idChnl2 = app.charIDToTypeID('Chnl')
    idMsk = app.charIDToTypeID('Msk ')
    ref20.putEnumerated(idChnl1, idChnl2, idMsk)
    desc38.putReference(idNull1, ref20)
    idMkVs = app.charIDToTypeID('MkVs')
    desc38.putBoolean(idMkVs, False)
    app.executeAction(idSlct, desc38, psDisplayNoDialogs)

    app.activeDocument.activeLayer.invert()



import os
import glob
from jinja2 import Template
import stringcase

template = Template(r"""
Usage
=====
{% for file_ in Examples.get_examples() %}
{{ Examples.get_name(file_) }}
{{ Examples.get_line(Examples.get_name(file_))}}
  .. literalinclude:: {{ Examples.convert_relative_path(file_) }}
{% endfor %}
    
""")


class Examples(object):
    def __init__(self, root):
        self._root = root

    def get_examples(self):
        return glob.glob(os.path.join(self._root, "*.py"))

    @staticmethod
    def convert_relative_path(file):
        path = file.split("examples")[1]
        return "../examples{}".format(path.replace("\\", "/"))

    @staticmethod
    def get_name(file):
        name = os.path.basename(file).split(".py")[0]
        return stringcase.titlecase(name)

    @staticmethod
    def get_line(name):
        return "-" * len(name)


if __name__ == '__main__':
    root = os.path.dirname(os.path.dirname(__file__))
    with open(os.path.join(root, "docs", "usage.rst"), "w") as f:
        f.write(template.render(
            Examples=Examples(os.path.join(root, "examples"))))



"""Add slate information dynamically。"""

from datetime import datetime
import os

import photoshop as ps

app = ps.Application()
app.load(os.path.join(os.path.dirname(__file__), "slate_template.psd"))
layer_set = app.activeDocument.layerSets.getByName("template")

data = {
    "project name": "test_project",
    "datetime": datetime.today().strftime('%Y-%m-%d')
}
for layer in layer_set.layers:
    if layer.kind == "TextLayer":
        layer.textItem.contents = data[layer.textItem.contents.strip()]

doc = app.activeDocument

jpg_file = "d:/photoshop_slate.jpg"

doc.saveAs(jpg_file, ps.JPEGSaveOptions())

doc.close()

os.startfile(jpg_file)



"""Add slate information dynamically。"""

from photoshop import Session


with Session() as adobe:
    doc = adobe.app.documents.add(2000, 2000)
    text_color = adobe.SolidColor()
    text_color.rgb.red = 255
    new_text_layer = doc.artLayers.add()
    new_text_layer.kind = adobe.LayerKind.TextLayer
    new_text_layer.textItem.contents = 'Hello, World!'
    new_text_layer.textItem.position = [160, 167]
    new_text_layer.textItem.size = 40
    new_text_layer.textItem.color = text_color
    options = adobe.JPEGSaveOptions(quality=1)
    jpg = 'd:/hello_world.jpg'
    doc.saveAs(jpg, options, asCopy=True)
    adobe.app.doJavaScript(f'alert("save to jpg: {jpg}")')



from photoshop._core import Photoshop


class TextItem(Photoshop):
    object_name = 'Application'

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def contents(self):
        """str:"""
        return self.app.contents

    @contents.setter
    def contents(self, text):
        self.app.contents = text

    @property
    def position(self):
        return self.app.position

    @position.setter
    def position(self, array):
        """The position of the origin for the text.

        The array must contain two values. Setting this property is basically
        equivalent to clicking the text tool at a point in the documents to
        create the point of origin for text.

        """
        self.app.position = array

    @property
    def size(self):
        return self.app.size

    @size.setter
    def size(self, value):
        self.app.size = value

    @property
    def color(self):
        return self.app.color

    @color.setter
    def color(self, color_value):
        """

        Args:
            color_value (photoshop_python_api.SolidColor):

        Returns:

        """
        self.app.color = color_value

    def capitalization(self):
        return self.app.capitalization

    def alternateLigatures(self):
        return self.app.alternateLigatures

    @property
    def name(self):
        return self.app.name



"""Check whether the foreground is equal to the background color.

References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/CompareColors.py

"""
from photoshop import Session

with Session() as ps:
    if ps.app.foregroundColor.isEqual(ps.app.backgroundColor):
        ps.echo("They're Equal.")
    else:
        ps.echo("NOT Equal.")



import photoshop as ps

# Get photoshop instance.
app = ps.Application()

# Add new document and set name to "Example for move to End."
active_document = app.documents.add(name="Example for move to End.")

# Add a new layer set.
group_layer = active_document.layerSets.add()
# Add a layer in the group.
layer = group_layer.artLayers.add()
layer.name = "This is a child layer."
# Add a new layer in this active document top.
top_layer = active_document.artLayers.add()
top_layer.name = "This is a top layer."
top_layer.moveToEnd(group_layer)



"""This class provides all photoshop API core functions."""
import winreg
import os

from comtypes.client import CreateObject
from photoshop import constants
from photoshop.errors import PhotoshopPythonAPIError
import platform


class Photoshop(object):
    _root = 'Photoshop'
    REG_PATH = 'SOFTWARE\\Adobe\\Photoshop'
    _object_name = 'Application'
    object_name = None

    def __init__(self, ps_version=None, parent=None):
        self._program_name = None
        version_mappings = constants.PHOTOSHOP_VERSION_MAPPINGS
        self.photoshop_version = os.getenv('PS_VERSION', ps_version)
        self.app_id = version_mappings.get(self.photoshop_version,
                                           self._get_program_id())
        try:
            self.app = self.instance_app(self.app_id)
        except OSError:
            try:
                self.app = self.instance_app(self._get_program_id())
            except OSError:
                raise PhotoshopPythonAPIError(
                    'Please check if you have '
                    'Photoshop installed correctly.',
                )
        if parent:
            self.adobe = self.app
            self.app = parent

    @property
    def typename(self):
        return self.__class__.__name__

    def __call__(self, *args, **kwargs):
        return self.app

    def __str__(self):
        return f'{self.__class__.__name__} <{self._program_name}>'

    def __repr__(self):
        return self

    def __getattribute__(self, item):
        try:
            return super().__getattribute__(item)
        except AttributeError:
            return getattr(self.app, item)

    @staticmethod
    def open_key(key):
        """Open the register key.

        Args:
            key (str): The key of register.

        Returns:
            str: The handle to the specified key.

        """
        machine_type = platform.machine()
        mappings = {
            'AMD64': winreg.KEY_WOW64_64KEY
        }
        return winreg.OpenKey(
            winreg.HKEY_LOCAL_MACHINE,
            key,
            access=winreg.KEY_READ | mappings.get(machine_type,
                                                  winreg.KEY_WOW64_32KEY)
        )

    def get_application_path(self):
        """str: The absolute path of Photoshop installed location."""
        key = self.open_key(f'{self.REG_PATH}\\{self.get_program_id()}')
        return winreg.QueryValueEx(key, 'ApplicationPath')[0]

    def get_plugin_path(self):
        """str: The absolute plugin path of Photoshop."""
        return os.path.join(self.get_application_path(), 'Plug-ins')

    def get_presets_path(self):
        """str: The absolute presets path of Photoshop."""
        return os.path.join(self.get_application_path(), 'Presets')

    def get_script_path(self):
        """str: The absolute scripts path of Photoshop."""
        return os.path.join(self.get_presets_path(), 'Scripts')

    def instance_app(self, ps_id):
        naming_space = [self._root]
        if not self.object_name:
            naming_space.append(self._object_name)
        else:
            naming_space.append(self.object_name)
        naming_space.append(ps_id)
        self._program_name = self._assemble_program_name(naming_space)
        return CreateObject(self._program_name, dynamic=True)

    def get_program_id(self):
        key = self.open_key(self.REG_PATH)
        return winreg.EnumKey(key, 0)

    def _get_program_id(self):
        self.app_id = self.get_program_id().split('.')[0]
        return self.app_id

    @staticmethod
    def _assemble_program_name(names):
        """Assemble program name of Photoshop.

        Args:
            names (list of str): The name to be assembled.
                .e.g:
                    [
                        'Photoshop',
                        'ActionDescriptor',
                        '140'
                    ]

        Returns:
            str: Assembled name.

        Examples:
            Photoshop.ActionDescriptor
            Photoshop.ActionDescriptor.140
            Photoshop.ActionList
            Photoshop.ActionList.140
            Photoshop.ActionReference
            Photoshop.ActionReference.140
            Photoshop.Application
            Photoshop.Application.140
            Photoshop.BatchOptions
            Photoshop.BatchOptions.140
            Photoshop.BitmapConversionOptions
            Photoshop.BMPSaveOptions
            Photoshop.BMPSaveOptions.140
            Photoshop.CameraRAWOpenOptions
            Photoshop.CameraRAWOpenOptions.140

        """
        return '.'.join(names)

    def eval_javascript(self, javascript, Arguments=None, ExecutionMode=None):
        return self.adobe.doJavaScript(javascript, Arguments, ExecutionMode)



from photoshop.artlayer import ArtLayer


def hello_world():

    artlayer = ArtLayer()

    # attrs = [
    #     # "build",
    #     "colorSettings",
    #     "currentTool",
    #     "displayDialogs",
    #     "documents.name"
    # ]
    # for attr in attrs:
    #     print("app.{}".format(attr), _getattr(app, attr))
    print(artlayer.fillOpacity)


if __name__ == '__main__':
    hello_world()



import photoshop as ps
from photoshop import Session

# style 1
app = ps.Application()
app.load("your/psd/or/psb/file_path.psd")

# style 2
with Session("your/psd/or/psb/file_path.psd", action="open") as ps:
    ps.echo(ps.active_document.name)



import os
import glob

root = os.path.join(os.path.dirname(os.path.dirname(__file__)), "examples")

for script in glob.glob(os.path.join(root, "*.py")):
    try:
        exec(open(script, "r").read())
    except:
        print("Test failed: {}".format(script))



try:
    from photoshop.action_descriptor import ActionDescriptor
    from photoshop.action_refrence import ActionReference
    from photoshop.application import Application
    from photoshop.artlayer import ArtLayer
    from photoshop.layerSets import LayerSets
    from photoshop.layerSet import LayerSet
    from photoshop.colors import (
        LabColor,
        HSBColor,
        CMYKColor,
        RGBColor,
    )
    from photoshop.document import Document
    from photoshop.documents import Documents
    from photoshop.layer import Layer
    from photoshop.save_options import (
        GIFSaveOptions,
        JPEGSaveOptions,
        PDFSaveOptions,
        PNGSaveOptions,
    )
    from photoshop.solid_color import SolidColor
    from photoshop.text_item import TextItem
    from photoshop.enumerations import NoiseDistribution
    from photoshop.enumerations import Units
    from photoshop.enumerations import LayerKind
    from photoshop.enumerations import NewDocumentMode
    from photoshop.enumerations import DocumentFill
    from photoshop.enumerations import DialogModes
    from photoshop.enumerations import SelectionType
    from photoshop.enumerations import TextureType
    from photoshop.enumerations import ColorBlendMode
    from photoshop.enumerations import StrokeLocation
    from photoshop.enumerations import SaveOptions
    from photoshop.constants import *
    from photoshop.errors import PhotoshopPythonAPIError
    from photoshop.errors import COMError
except ModuleNotFoundError:
    # Fix Build docs failed on readthedocs.
    pass


class Session:
    """Session of photoshop.

    We can control active documents in this Session.

    """
    presetKind = 'presetKindType'
    smartSharpen = 'smartSharpen'
    presetKindType = 'presetKindType'
    presetKindCustom = 'presetKindCustom'
    noiseReduction = 'noiseReduction'
    blur = 'blur'
    blurType = 'blurType'
    AMNT = 'Amnt'
    contentLayer = 'contentLayer'
    NULL = 'null'
    GSNB = 'GsnB'
    ORDN = 'Ordn'
    DOCI = 'DocI'
    CLS = 'Cls '  # This string ID requires a space.
    LYR = 'Lyr '  # This string ID requires a space.
    OFST = 'Ofst'
    PRC = '#Prc'
    RDS = 'Rds '  # This string ID requires a space.
    PX1 = '#Pxl'
    PLC = 'Plc '  # This string ID requires a space.
    IDNT = 'Idnt'
    HRZN = 'Hrzn'
    IN = 'In  '  # This string ID requires two spaces.
    TRGT = 'Trgt'
    SAVE = 'save'
    SAVE_STAGE = 'saveStage'
    SAVE_STAGE_TYPE = 'saveStageType'
    SAVE_SUCCEEDED = 'saveSucceeded'
    RASTERIZE_LAYER = 'rasterizeLayer'
    YSN = 'YsN '  # This string ID requires a space.
    N = 'N   '  # This string ID requires three spaces.
    VRTC = 'Vrtc'
    SVNG = 'Svng'
    FTCS = 'FTcs'
    QCST = 'QCSt'
    QCSA = 'Qcsa'
    FORCE_NOTIFY = 'forceNotify'
    JSMS = 'jsMs'
    JSCT = 'jsCt'
    PLACED_LAYER_EDIT_CONTENTS = 'placedLayerEditContents'

    def __init__(self, file_path=None, action=None, callback=None,
                 auto_close=False):
        super().__init__()
        self.path = file_path
        self._auto_close = auto_close
        self._callback = callback
        self._action = action
        self._active_document = None
        self.app = Application()
        self.SaveOptions = SaveOptions
        self.SolidColor = SolidColor
        self.StrokeLocation = StrokeLocation
        self.ColorBlendMode = ColorBlendMode
        self.TextItem = TextItem
        self.ActionReference = ActionReference()
        self.ActionDescriptor = ActionDescriptor()
        self.NoiseDistribution = NoiseDistribution
        self.TextureType = TextureType
        self.SelectionType = SelectionType
        self.DocumentFill = DocumentFill
        self.DialogModes = DialogModes
        self.NewDocumentMode = NewDocumentMode
        self.LayerKind = LayerKind
        self.Units = Units
        self.GIFSaveOptions = GIFSaveOptions
        self.JPEGSaveOptions = JPEGSaveOptions
        self.PDFSaveOptions = PDFSaveOptions
        self.PNGSaveOptions = PNGSaveOptions
        self.LabColor = LabColor
        self.HSBColor = HSBColor
        self.CMYKColor = CMYKColor
        self.RGBColor = RGBColor

    @property
    def active_document(self):
        try:
            if not self._active_document:
                return self.app.activeDocument
            return self._active_document
        except COMError:
            raise PhotoshopPythonAPIError("No active document available.")

    @staticmethod
    def echo(*args, **kwargs):
        """Print message."""
        print(*args, **kwargs)

    def alert(self, text: str):
        """Alert message box in photoshop.

        Args:
            text (str): The text will pop up in photoshop.

        """
        self.app.doJavaScript(f"alert('{text}')")

    @active_document.setter
    def active_document(self, active_document):
        """Set active document."""
        self._active_document = active_document

    def _action_open(self):
        self.active_document = self.app.open(self.path)

    def _action_new_document(self):
        self.active_document = self.app.documents.add()

    def _action_document_duplicate(self):
        self.active_document = self.active_document.duplicate()

    def __enter__(self):
        try:
            _action = getattr(self, '_action_{}'.format(self._action))
            _action()
        except AttributeError:
            pass
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            if self._callback:
                self._callback(self)
        except Exception as err:
            raise PhotoshopPythonAPIError(err)
        finally:
            if self._auto_close:
                self.active_document.close()


# All public APIs.
__all__ = [
    'ActionDescriptor',
    'ActionReference',
    'ArtLayer',
    'Application',
    'CMYKColor',
    'ColorBlendMode',
    'DialogModes',
    'DocumentFill',
    'Document',
    'Documents',
    'SelectionType',
    'StrokeLocation',
    'TextureType',
    'NewDocumentMode',
    'LayerKind',
    'SolidColor',
    'TextItem',
    'LabColor',
    'LayerSets',
    'LayerSet',
    'HSBColor',
    'RGBColor',
    'Layer',
    'NoiseDistribution',
    'SaveOptions',
    'JPEGSaveOptions',
    'PNGSaveOptions',
    'PDFSaveOptions',
    'GIFSaveOptions',
    'Units',
    'Session'
]

__title__ = 'photoshop_python_api'
__version__ = '0.7.2'
__author__ = 'Long Hao'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018 Long Hao'



from photoshop import Application


def hello_world():
    app = Application()

    # attrs = [
    #     # "build",
    #     "colorSettings",
    #     "currentTool",
    #     "displayDialogs",
    #     "documents.name"
    # ]
    # for attr in attrs:
    #     print("app.{}".format(attr), _getattr(app, attr))
    # print(app.build)
    # print(app.colorSettings)
    # print(app.currentTool)
    # print(app.displayDialogs)
    # print(app.macintoshFileTypes)
    # print(app.fonts)
    # print(app.foregroundColor.cmyk.black)
    # print(app.freeMemory)
    # print(app.locale)
    # print(app.measurementLog.exportMeasurements("d:/test.text"))
    # print(app.preferencesFolder)
    # print(app.recentFiles)
    # print(app.version)
    # print(app.beep())
    # print(app.bringToFront())
    # app.changeProgressText("a")
    # print(app.charIDToTypeID("jsCt"))
    # app.doForcedProgress("test", 'alert ("dasdads")')
    # app.doProgress("tes1t", 'alert ("aaaaaaaaaaaaaaaaaaaaaaaaaaaa")')
    # app.doProgressSegmentTask(50, 100, 100,
    #                           'alert("aaaaaaaaaaaaaaaaaaaaaaaaaaaa")')
    # app.doProgressSubTask(20, 100, 'alert("aaaaaaaaaaaaaaaaaaaaaaaaaaaa")')
    # app.doProgressTask(20, 'alert("aaaaaaaaaaaaaaaaaaaaaaaaaaaa")')

    # app.updateProgress(100, 2)
    # app.showColorPicker()
    # app.refreshFonts()
    app.purge(0)
    print(app.isQuicktimeAvailable())
    app.doJavaScript('alert', ['aasdasd'], 1)


if __name__ == '__main__':
    hello_world()



"""This scripts demonstrates how to rotate a layer 45 degrees clockwise.

References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/RotateLayer.py

"""

import photoshop as ps

app = ps.Application()

if len(app.documents) > 0:
    print(app.activeDocument.activeLayer.typename)
    if not app.activeDocument.activeLayer.isBackgroundLayer:
        docRef = app.activeDocument
        layerRef = docRef.layers[0]
        layerRef.rotate(45.0)
    else:
        print("Operation cannot be performed on background layer")
else:
    print("You must have at least one open document to run this script!")



# Import local modules
from photoshop._core import Photoshop


class TiffSaveOptions(Photoshop):
    object_name = 'TiffSaveOptions'

    def __int__(self):
        super().__init__()

    @property
    def alphaChannels(self):
        """If true, the alpha channels are saved."""
        return self.app.alphaChannels

    @alphaChannels.setter
    def alphaChannels(self, value):
        self.app.alphaChannels = value

    @property
    def annotations(self):
        """If true, the annotations are saved."""
        return self.app.annotations

    @annotations.setter
    def annotations(self, value):
        self.app.annotations = value

    @property
    def byteOrder(self):
        """The order in which the bytes will be read.
        Default:
        Mac OS when running in Mac OS, and IBM PC when running in Windows.
        """
        return self.app.byteOrder

    @byteOrder.setter
    def byteOrder(self, value):
        self.app.byteOrder = value

    @property
    def embedColorProfile(self):
        """If true, the color profile is embedded in the document."""
        return self.app.embedColorProfile

    @embedColorProfile.setter
    def embedColorProfile(self, value):
        self.app.embedColorProfile = value

    @property
    def imageCompression(self):
        """The compression type."""
        return self.app.imageCompression

    @imageCompression.setter
    def imageCompression(self, value):
        self.app.imageCompression = value

    @property
    def interleaveChannels(self):
        """If true, the channels in the image are interleaved."""
        return self.app.interleaveChannels

    @interleaveChannels.setter
    def interleaveChannels(self, value):
        self.app.interleaveChannels = value

    @property
    def jpegQuality(self):
        """The quality of the produced image, which is inversely proportionate
        to the amount of JPEG compression.
        Valid only for JPEG compressed TIFF documents. Range: 0 to 12.
        """
        return self.app.jpegQuality

    @jpegQuality.setter
    def jpegQuality(self, value):
        self.app.jpegQuality = value

    @property
    def layerCompression(self):
        return self.app.layerCompression

    @layerCompression.setter
    def layerCompression(self, value):
        """The method of compression to use when saving layers
        (as opposed to saving composite data).
        Valid only when `layers` = true.
        """
        self.app.layerCompression = value

    @property
    def layers(self):
        """If true, the layers are saved."""
        return self.app.layers

    @layers.setter
    def layers(self, value):
        self.app.layers = value

    @property
    def saveImagePyramid(self):
        """If true, preserves multi-resolution information."""
        return self.app.saveImagePyramid

    @saveImagePyramid.setter
    def saveImagePyramid(self, value):
        self.app.saveImagePyramid = value

    @property
    def spotColors(self):
        """If true, spot colors are saved."""
        return self.app.spotColors

    @spotColors.setter
    def spotColors(self, value):
        self.app.spotColors = value

    @property
    def transparency(self):
        return self.app.transparency

    @transparency.setter
    def transparency(self, value):
        """If true, saves the transparency as an additional alpha channel when
         the file is opened in another application."""
        self.app.transparency = value



import os
import time
from photoshop._core import Photoshop
from photoshop.active_document import ActiveDocument
from photoshop.documents import Documents
from photoshop.preferences import Preferences
from photoshop.solid_color import SolidColor


class Application(Photoshop):

    def __init__(self, version=None):
        super().__init__(ps_version=version)

    @property
    def activeDocument(self):
        """The frontmost documents."""
        return ActiveDocument(self.app.activeDocument)

    @property
    def backgroundColor(self):
        return SolidColor(self.app.backgroundColor)

    @backgroundColor.setter
    def backgroundColor(self, color):
        self.app.backgroundColor = color

    @property
    def build(self):
        return self.app.build

    @property
    def colorSettings(self):
        return self.app.colorSettings

    @property
    def currentTool(self):
        return self.app.currentTool

    @currentTool.setter
    def currentTool(self, tool_name):
        self.app.currentTool = tool_name

    @property
    def displayDialogs(self):
        """The dialog mode for the document, which indicates whether or not
        Photoshop displays dialogs when the script runs."""
        return self.app.displayDialogs

    @displayDialogs.setter
    def displayDialogs(self, value):
        self.app.displayDialogs = value

    @property
    def documents(self):
        return Documents(self.app.documents)

    # TODO:
    @property
    def fonts(self):
        return self.app.fonts

    @property
    def foregroundColor(self):
        """The default foreground color (used to paint, fill,
        and stroke selections)."""
        return SolidColor(parent=self.app.foregroundColor)

    @property
    def freeMemory(self):
        """The amount of unused memory available to Photoshop."""
        return self.app.freeMemory

    @property
    def locale(self):
        """The language locale of the application."""
        return self.app.locale

    @property
    def macintoshFileTypes(self):
        """A list of the image file types Photoshop can open."""
        return self.app.macintoshFileTypes

    @property
    def measurementLog(self):
        """The log of measurements taken."""
        return self.app.measurementLog

    @property
    def name(self):
        return self.app.name

    @property
    def notifiers(self):
        """The notifiers currently configured (in the Scripts Events Manager
        menu in the application)."""
        return self.app.notifiers

    @property
    def notifiersEnabled(self):
        """If true, notifiers are enabled."""
        return self.app.notifiersEnabled

    @property
    def parent(self):
        """The object’s container."""
        return self.app.parent

    @property
    def path(self):
        return self.app.path

    @property
    def playbackDisplayDialogs(self):
        return self.app.playbackDisplayDialogs

    @property
    def playbackParameters(self):
        return self.app.playbackParameters

    @property
    def preferences(self):
        return Preferences(self.app.preferences)

    @property
    def preferencesFolder(self):
        return self.app.preferencesFolder

    @property
    def recentFiles(self):
        return self.app.recentFiles

    @property
    def scriptingBuildDate(self):
        return self.app.scriptingBuildDate

    @property
    def scriptingVersion(self):
        return self.app.scriptingVersion

    @property
    def systemInformation(self):
        return self.app.systemInformation

    @property
    def typename(self):
        return self.app.typename

    @property
    def version(self):
        return self.app.version

    @property
    def windowsFileTypes(self):
        return self.app.windowsFileTypes

    # Methods.

    def batch(self):
        pass

    def beep(self):
        """Alerts the user."""
        pass

    def bringToFront(self):
        return self.eval_javascript('app.bringToFront();')

    def changeProgressText(self, text):
        """Changes the text that appears in the progress window."""
        self.eval_javascript(f"app.changeProgressText('{text}');")

    def doAction(self, action, action_from):
        """Plays the specified action from the Actions palette."""
        return self.app.doAction(action, action_from)

    def doForcedProgress(self, title, javascript):
        script = "app.doForcedProgress('{}', '{}')".format(
            title,
            javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgress(self, title, javascript):
        """Performs a task with a progress bar. Other progress APIs must be
        called periodically to update the progress bar and allow cancelling.

        Args:
            title (str): String to show in the progress window.
            javascript (str): JavaScriptString to execute.

        """
        script = "app.doProgress('{}', '{}')".format(
            title,
            javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgressSegmentTask(self, segmentLength, done, total, javascript):
        script = "app.doProgressSegmentTask({}, {}, {}, '{}');".format(
            segmentLength, done, total, javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgressSubTask(self, index, limit, javascript):
        script = "app.doProgressSubTask({}, {}, '{}');".format(
            index, limit, javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgressTask(self, index, javascript):
        """Sections off a portion of the unused progress bar for execution of
        a subtask. Returns false on cancel.

        """
        script = f"app.doProgressTask({index}, '{javascript}');"
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def eraseCustomOptions(self):
        pass

    def executeAction(self, eventID, descriptor, displayDialogs=2):
        return self.app.executeAction(eventID, descriptor, displayDialogs)

    @property
    def activeLayer(self):
        return self.app.ArtLayer

    @property
    def layerSets(self):
        return self.app.LayerSets

    def open(self, document_file_path):
        return ActiveDocument(self.app.open(document_file_path))

    def load(self, document_file_path):
        """Loads a support document."""
        self.app.load(document_file_path)
        return self.activeDocument

    def doJavaScript(self, javascript, Arguments=None, ExecutionMode=None):
        return self.app.doJavaScript(javascript, Arguments, ExecutionMode)

    def isQuicktimeAvailable(self):
        return self.app.IsQuicktimeAvailable()

    def purge(self, index):
        """

        Args:
            index:
                .e.g:
                    0: Clears all caches.
                    1: Clears the clipboard.
                    2: Deletes all history states from the History palette.
                    3: Clears the undo cache.

        Returns:

        """
        self.app.purge(index)

    def refreshFonts(self):
        """Force the font list to get refreshed."""
        return self.eval_javascript('app.refreshFonts();')

    def showColorPicker(self):
        return self.eval_javascript('app.showColorPicker();')

    @staticmethod
    def system(command):
        os.system(command)

    def stringIDToTypeID(self, string):
        return self.app.stringIDToTypeID(string)

    def typeIDToStringID(self, typeID):
        return self.app.typeIDToStringID(typeID)

    def typeIDToCharID(self, typeID):
        return self.app.typeIDToCharID(typeID)

    def updateProgress(self, done, total):
        self.eval_javascript(f'app.updateProgress({done}, {total})')



from photoshop import Session

with Session() as ps:
    # Print the current tool.
    ps.echo(ps.app.currentTool)

    # Set current tool to `typeCreateOrEditTool`.
    ps.app.currentTool = "typeCreateOrEditTool"



from photoshop.colors.cmyk import CMYKColor
from photoshop.colors.hsb import HSBColor
from photoshop.colors.lab import LabColor
from photoshop.colors.rgb import RGBColor

__all__ = [
    'LabColor',
    'RGBColor',
    'CMYKColor',
    'HSBColor',
]



# Fill the current selection with an RGB color.


from photoshop import Session

with Session() as ps:
    start_ruler_units = ps.app.Preferences.RulerUnits

    if len(ps.app.documents) < 1:
        if start_ruler_units is not ps.Units.Pixels:
            ps.app.Preferences.RulerUnits = ps.Units.Pixels
        docRef = ps.app.documents.add(320, 240, 72, None,
                                      ps.NewDocumentMode.NewRGB,
                                      ps.DocumentFill.White)
        docRef.artLayers.add()
        ps.app.preferences.rulerUnits = start_ruler_units

    if not ps.active_document.activeLayer.isBackgroundLayer:
        selRef = ps.active_document.selection
        fillcolor = ps.SolidColor()
        fillcolor.rgb.red = 225
        fillcolor.rgb.green = 0
        fillcolor.rgb.blue = 0
        selRef.fill(fillcolor, ps.ColorBlendMode.NormalBlendColor, 25, False)
    else:
        ps.echo("Can't perform operation on background layer.")



# Import local modules
from photoshop._core import Photoshop


class ExportOptionsSaveForWeb(Photoshop):
    object_name = 'ExportOptionsSaveForWeb'

    def __init__(self):
        super().__init__()
        self.format = 13  # PNG
        self.PNG8 = False  # Sets it to PNG-24 bit

    @property
    def PNG8(self):
        return self.app.PNG8

    @PNG8.setter
    def PNG8(self, value):
        self.app.PNG8 = value

    @property
    def blur(self):
        return self.app.blur

    @blur.setter
    def blur(self, value):
        self.app.blur = value

    @property
    def colorReduction(self):
        return self.app.colorReduction

    @colorReduction.setter
    def colorReduction(self, value):
        self.app.colorReduction = value

    @property
    def colors(self):
        """The number of colors in the palette."""
        return self.app.colors

    @colors.setter
    def colors(self, value):
        """The number of colors in the palette."""
        self.app.colors = value

    @property
    def dither(self):
        return self.app.dither

    @dither.setter
    def dither(self, value):
        self.app.dither = value


class PNGSaveOptions(Photoshop):
    object_name = 'PNGSaveOptions'

    def __init__(self):
        super().__init__()
        self.interlaced = False
        self.compression = True

    @property
    def interlaced(self):
        return self.app.interlaced

    @interlaced.setter
    def interlaced(self, value):
        self.app.interlaced = value



"""
References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/CopyAndPaste.py

"""

import photoshop as ps

app = ps.Application()

startRulerUnits = app.preferences.rulerUnits

app.preferences.rulerUnits = ps.Units.Inches

doc = app.documents.add(7, 5, 72, None,
                        ps.NewDocumentMode.NewRGB, ps.DocumentFill.White)

# Make sure the active layer is not a text layer, which cannot be copied to the
# clipboard.
if doc.activeLayer.kind != ps.LayerKind.TextLayer:
    # Select the left half of the document. Selections are always expressed
    # in pixels regardless of the current ruler unit type, so we're computing
    # the selection corner points based on the inch unit width and height
    # of the document
    x2 = (doc.width * doc.resolution) / 2
    y2 = doc.height * doc.resolution

    sel_area = ((0, 0), (x2, 0), (x2, y2), (0, y2))
    doc.selection.select(sel_area, ps.SelectionType.ReplaceSelection, 0, False)

    doc.selection.copy()

    # The new doc is created
    # need to change ruler units to pixels because x2 and y2 are pixel units.
    app.preferences.rulerUnits = ps.Units.Pixels
    pasteDoc = app.documents.add(x2, y2, doc.resolution, "Paste Target")
    pasteDoc.paste()
else:
    print("You cannot copy from a text layer")

if startRulerUnits != app.preferences.rulerUnits:
    app.preferences.rulerUnits = startRulerUnits




import photoshop as ps

app = ps.Application()

start_ruler_units = app.preferences.rulerUnits

if len(app.documents) < 1:
    if start_ruler_units is not ps.Units.Pixels:
        app.preferences.rulerUnits = ps.Units.Pixels
    docRef = app.documents.add(
        320, 240, 72, None, ps.NewDocumentMode.NewRGB,
        ps.DocumentFill.BackgroundColor,
    )
else:
    docRef = app.activeDocument

layerRef = docRef.artLayers.add()
layerRef2 = docRef.artLayers.add()
layerRef.link(layerRef2)

# Set the ruler back to where it was
app.preferences.rulerUnits = start_ruler_units



import photoshop as ps


def hello_world():
    app = ps.Application()
    doc = app.documents.add()
    text_color = ps.SolidColor()
    text_color.rgb.green = 255
    new_text_layer = doc.artLayers.add()
    new_text_layer.kind = ps.LayerKind.TextLayer
    new_text_layer.textItem.contents = 'Hello, World!'
    new_text_layer.textItem.position = [160, 167]
    new_text_layer.textItem.size = 40
    new_text_layer.textItem.color = text_color
    options = ps.JPEGSaveOptions(quality=5)
    jpg = 'd:/hello_world.jpg'
    doc.saveAs(jpg, options, asCopy=True)
    app.doJavaScript(f'alert("save to jpg: {jpg}")')


if __name__ == '__main__':
    hello_world()



from photoshop._core import Photoshop
from photoshop.layers import Layers
from photoshop.artlayers import ArtLayers


class LayerSet(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def name(self):
        return self.app.name

    @property
    def artLayers(self):
        return ArtLayers(self.app.artLayers)

    @property
    def enabledChannels(self):
        return self.app.enabledChannels

    @property
    def layerSets(self):
        return self.app.layerSets

    @property
    def layers(self):
        return Layers(self.app.layers)

    @property
    def parent(self):
        return self.app.parent

    def add(self):
        self.app.add()

    def merge(self):
        self.app.merge()

    def __iter__(self):
        for layer in self.app:
            yield layer

    @property
    def isBackgroundLayer(self):
        index = "app.activeDocument.layers.length-1"
        layers = "app.activeDocument.layers"
        layer = self.eval_javascript(f"{layers}({index}).name")
        return bool(layer == self.app.name)



from photoshop._core import Photoshop
from photoshop.artlayer import ArtLayer
from photoshop.layer import Layer


class Layers(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def _layers(self):
        return list(self.app)

    def __len__(self):
        return self.length

    def __getitem__(self, key):
        item = self._layers[key]
        try:
            # If have text item will be return ArtLayer.
            item.textItem
            return ArtLayer(item)
        except:
            return Layer(self._layers[key])

    @property
    def length(self):
        return len(self._layers)

    def removeAll(self):
        return [layer.name for layer in self.app]

    def item(self, index):
        return ArtLayer(self.app.item(index))

    def __iter__(self):
        for layer in self._layers:
            if layer.kind == "1":
                yield Layer(layer)
            else:
                yield ArtLayer(layer)



# Set the active layer to the last art layer of the active document, or the
# first if the last is already active.

from photoshop import Session

with Session() as ps:
    if len(ps.app.documents) < 1:
        docRef = ps.app.documents.add()
    else:
        docRef = ps.app.activeDocument

    if len(docRef.layers) < 2:
        docRef.artLayers.add()

    print(docRef.activeLayer.name)
    print(docRef.layers.item(len(docRef.layers)))
    new_layer = docRef.artLayers.add()
    print(new_layer.name)
    new_layer.name = "test"



"""This script demonstrates how you can use the action manager to execute the
Emboss filter.

References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/SmartSharpen.py

"""

import os
import photoshop as ps


app = ps.Application()

fileName = os.path.join(os.path.dirname(__file__), 'layer_comps.psd')
docRef = app.open(fileName)

nlayerSets = docRef.layerSets
nArtLayers = docRef.layerSets.item(nlayerSets.length)
docRef.activeLayer = nArtLayers.artLayers.item(nArtLayers.artLayers.length)


def SmartSharpen(inAmount, inRadius, inNoise):
    idsmart_sharpen_id = app.stringIDToTypeID(ps.smartSharpen)
    desc37 = ps.ActionDescriptor()

    idpresetKind = app.stringIDToTypeID(ps.presetKind)
    idpresetKindType = app.stringIDToTypeID(ps.presetKindType)
    idpresetKindCustom = app.stringIDToTypeID(ps.presetKindCustom)
    desc37.putEnumerated(idpresetKind, idpresetKindType, idpresetKindCustom)

    idAmnt = app.charIDToTypeID(ps.AMNT)
    idPrc = app.charIDToTypeID(ps.RDS)
    desc37.putUnitDouble(idAmnt, idPrc, inAmount)

    idRds = app.charIDToTypeID(ps.RDS)
    idPxl = app.charIDToTypeID(ps.PX1)
    desc37.putUnitDouble(idRds, idPxl, inRadius)

    idnoiseReduction = app.stringIDToTypeID(ps.noiseReduction)
    idPrc = app.charIDToTypeID(ps.PRC)
    desc37.putUnitDouble(idnoiseReduction, idPrc, inNoise)

    idblur = app.charIDToTypeID(ps.blur)
    idblurType = app.stringIDToTypeID(ps.blurType)
    idGsnB = app.charIDToTypeID(ps.GSNB)
    desc37.putEnumerated(idblur, idblurType, idGsnB)

    app.ExecuteAction(idsmart_sharpen_id, desc37)


SmartSharpen(300, 2.0, 20)



import os
from photoshop import Session

with Session(auto_close=True) as ps:
    orig_name = ps.active_document.name
    width_str = ps.active_document.width
    height_str = ps.active_document.height
    thumb_name = f'{orig_name}_tumb'
    index = width_str / 1248

    thumb_width = int(width_str / index)

    thumb_height = int(height_str / index)

    thumb_doc = ps.active_document.duplicate(thumb_name)
    thumb_doc.resizeImage(thumb_width, thumb_height - 100)
    thumb_file = 'd:/thumb.jpg'
    thumb_doc.saveAs(thumb_file, ps.JPEGSaveOptions(), asCopy=True)
    thumb_doc.close()
    os.startfile(thumb_file)



"""The photoshop enumerations."""


def magic_attr(cls):
    def _mappings(index):
        for key, value in cls.__dict__.items():
            if index == value:
                return key

    class PatchCls(cls):
        def __init__(self, index):
            self._index = index

        def __repr__(self):
            return _mappings(self._index)

        def __eq__(self, other):
            return _mappings(self._index) == other

    return PatchCls


@magic_attr
class AdjustmentReference:
    Relative = 1
    Absolute = 2


@magic_attr
class AnchorPosition:
    TopLeft = 1
    TopCenter = 2
    TopRight = 3
    MiddleLeft = 4
    MiddleCenter = 5
    BottomLeft = 7
    BottomCenter = 8
    BottomRight = 9


@magic_attr
class AntiAlias:
    NoAntialias = 1
    Sharp = 2
    Crisp = 3
    Strong = 4
    Smooth = 5


@magic_attr
class NewDocumentMode:
    NewGray = 1
    NewRGB = 2
    NewCMYK = 3
    NewLab = 4
    NewBitmap = 5


@magic_attr
class DocumentFill:
    White = 1
    BackgroundColor = 2
    Transparent = 3


@magic_attr
class DialogModes:
    DisplayAllDialogs = 1
    DisplayErrorDialogs = 2
    DisplayNoDialogs = 3


@magic_attr
class SelectionType:
    ReplaceSelection = 1
    ExtendSelection = 2
    DiminishSelection = 3
    IntersectSelection = 4


@magic_attr
class TextureType:
    BlocksTexture = 1
    CanvasTexture = 2
    FrostedTexture = 3
    TinyLensTexture = 4
    TextureFile = 5


@magic_attr
class Units:
    Pixels = 1
    Inches = 2
    CM = 3
    MM = 4
    Points = 5
    Picas = 6
    Percent = 7


@magic_attr
class LayerKind:
    NormalLayer = 1
    TextLayer = 2
    SolidFillLayer = 3


@magic_attr
class FormatOptionsType:
    OptimizedBaseline = 2
    Progressive = 3
    StandardBaseline = 4


@magic_attr
class MatteType:
    NoMatte = 1
    ForegroundColorMatte = 2
    BackgroundColorMatte = 3
    WhiteMatte = 4
    BlackMatte = 5
    SemiGray = 6
    NetscapeGrayMatte = 7


@magic_attr
class ExtensionType:
    Lowercase = 2
    Uppercase = 3


@magic_attr
class BitsPerChannelType:
    Document1Bit = 1
    Document8Bits = 8
    Document16Bits = 16
    Document32Bits = 32


@magic_attr
class ColorBlendMode:
    NormalBlendColor = 2
    DissolveBlend = 3
    DarkenBlend = 4
    MultiplyBlend = 5
    ColorBurnBlend = 6
    LinearBurnBlend = 7
    LightenBlend = 8
    ScreenBlend = 9
    ColorDodgeBlend = 10
    LinearDodgeBlend = 11
    OverlayBlend = 12
    SoftLightBlend = 13
    HardLightBlend = 14
    VividLightBlend = 15
    LinearLightBlend = 16
    PinLightBlend = 17
    DifferenceBlend = 18
    ExclusionBlend = 19
    HueBlend = 20
    SaturationBlendColor = 21
    ColorBlendMode = 22
    LuminosityBlend = 23
    BehindBlend = 24
    ClearBlend = 25
    HardMixBlend = 26
    LighterColorBlend = 27
    DarkerColorBlend = 28
    SubtractBlend = 29
    DivideBlend = 30


@magic_attr
class StrokeLocation:
    InsideStroke = 1
    CenterStroke = 2
    OutsideStroke = 3


@magic_attr
class NoiseDistribution:
    UniformNoise = 1
    GaussianNoise = 2


@magic_attr
class SaveOptions:
    SaveChanges = 1
    DoNotSaveChanges = 2
    PromptToSaveChanges = 3



from photoshop._core import Photoshop


class LabColor(Photoshop):

    def __init__(self, parent):
        super().__init__(parent=parent)
        self.A = 0
        self.b = 0
        self.L = 0

    @property
    def A(self):
        return self.app.A

    @A.setter
    def A(self, value):
        self.app.A = value

    @property
    def B(self):
        return self.app.B

    @B.setter
    def B(self, value):
        print(value)
        self.app.B = value

    @property
    def L(self):
        return self.app.L

    @L.setter
    def L(self, value):
        self.app.L = value



from photoshop._core import Photoshop


class Preferences(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def rulerUnits(self):
        return self.app.rulerUnits

    @rulerUnits.setter
    def rulerUnits(self, value):
        self.app.rulerUnits = value



from photoshop._core import Photoshop


class RGBColor(Photoshop):

    def __init__(self, parent):
        super().__init__(parent=parent)
        self.blue = 0
        self.red = 0
        self.green = 0

    @property
    def blue(self):
        return self.app.blue

    @blue.setter
    def blue(self, value: int):
        self.app.blue = value

    @property
    def green(self):
        return self.app.green

    @green.setter
    def green(self, value: int):
        self.app.green = value

    @property
    def red(self):
        return self.app.red

    @red.setter
    def red(self, value: int):
        self.app.red = value

    @property
    def hexValue(self):
        return self.app.hexValue

    @hexValue.setter
    def hexValue(self, value):
        self.app.hexValue = value



import photoshop as ps

app = ps.Application()
jsx = r"""
var doc = app.activeDocument;
var orig_name = doc.name;
alert(orig_name);
"""
app.doJavaScript(jsx)


# Print name of current active document.
print(app.doJavaScript("app.activeDocument.name"))



# Create a new Photoshop document with diminsions 4 inches by 4 inches.
import photoshop as ps

# Start up Photoshop application
app = ps.Application()

start_ruler_units = app.preferences.rulerUnits

app.preferences.rulerUnits = ps.Units.Pixels

# Create the document
docRef = app.documents.add(1920, 1080, 72.0, "My New Document")

# Make sure to set the ruler units prior to creating the document.
app.preferences.rulerUnits = start_ruler_units



"""This script demonstrates how you can use the action manager to execute the
Emboss filter.

References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/SmartSharpen.py

"""

import os
from photoshop import Session

fileName = os.path.join(os.path.dirname(__file__), 'layer_comps.psd')

with Session(fileName) as ps:
    def SmartSharpen(inAmount, inRadius, inNoise):
        idsmart_sharpen_id = ps.app.stringIDToTypeID(ps.smartSharpen)
        desc37 = ps.ActionDescriptor()

        idpresetKind = ps.app.stringIDToTypeID(ps.presetKind)
        idpresetKindType = ps.app.stringIDToTypeID(ps.presetKindType)
        idpresetKindCustom = ps.app.stringIDToTypeID(ps.presetKindCustom)
        desc37.putEnumerated(idpresetKind, idpresetKindType,
                             idpresetKindCustom)

        idAmnt = ps.app.charIDToTypeID(ps.AMNT)
        idPrc = ps.app.charIDToTypeID(ps.RDS)
        desc37.putUnitDouble(idAmnt, idPrc, inAmount)

        idRds = ps.app.charIDToTypeID(ps.RDS)
        idPxl = ps.app.charIDToTypeID(ps.PX1)
        desc37.putUnitDouble(idRds, idPxl, inRadius)

        idnoiseReduction = ps.app.stringIDToTypeID(ps.noiseReduction)
        idPrc = ps.app.charIDToTypeID(ps.PRC)
        desc37.putUnitDouble(idnoiseReduction, idPrc, inNoise)

        idblur = ps.app.charIDToTypeID(ps.blur)
        idblurType = ps.app.stringIDToTypeID(ps.blurType)
        idGsnB = ps.app.charIDToTypeID(ps.GSNB)
        desc37.putEnumerated(idblur, idblurType, idGsnB)

        ps.app.ExecuteAction(idsmart_sharpen_id, desc37)


    SmartSharpen(300, 2.0, 20)



"""Create a stroke around the current selection, Set the stroke color and
width of the new stroke.

References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/SelectionStroke.py

"""

import photoshop as ps

app = ps.Application()

if len(list((i, x) for i, x in enumerate(app.documents, 1))) > 0:
    if not app.activeDocument.activeLayer.isBackgroundLayer:
        psPixels = 1
        start_ruler_units = app.Preferences.RulerUnits
        app.preferences.rulerUnits = ps.Units.Pixels

        selRef = app.activeDocument.selection
        offset = 10
        selBounds = ((offset, offset),
                     (app.activeDocument.width - offset, offset),
                     (app.activeDocument.width - offset,
                      app.activeDocument.height - offset),
                     (offset, app.activeDocument.height - offset))

        selRef.select(selBounds)
        selRef.selectBorder(5)

        # create text color properties
        strokeColor = ps.SolidColor()

        strokeColor.cmyk.cyan = 58
        strokeColor.cmyk.magenta = 0
        strokeColor.cmyk.yellow = 70
        strokeColor.cmyk.black = 0
        app.displayDialogs = ps.DialogModes.DisplayNoDialogs
        selRef.stroke(strokeColor, 2, ps.StrokeLocation.OutsideStroke,
                      ps.ColorBlendMode.ColorBlendMode, 75, True)

        # Set ruler units back the way we found it.
        app.preferences.rulerUnits = start_ruler_units
    else:
        print("Operation cannot be performed on background layer")
else:
    print("Create a document with an active selection before running this "
          "script!")



"""A record of key-value pairs for actions, such as those included on the
Adobe Photoshop Actions menu. The ActionDescriptor class is part of the Action
Manager functionality. For more details on the Action Manager,
see the Photoshop Scripting Guide."""
from photoshop._core import Photoshop


class ActionDescriptor(Photoshop):
    object_name = 'ActionDescriptor'

    def __init__(self):
        super().__init__()

    @property
    def count(self):
        return self.app.count

    def clear(self):
        self.app.clear()

    def erase(self, key):
        return self.app.erase(key)

    def getBoolean(self, key):
        return self.app.getBoolean(key)

    def getClass(self, key):
        return self.app.getClass(key)

    def getDouble(self, key):
        return self.app.getDouble(key)

    def getEnumerationType(self, index):
        return self.app.getEnumerationType(index)

    def getEnumerationValue(self, index):
        return self.app.getEnumerationValue(index)

    def getInteger(self, index):
        return self.app.getInteger(index)

    def getLargeInteger(self, index):
        return self.app.getLargeInteger(index)

    def getList(self, index):
        return self.app.getList(index)

    def getTypeObject(self, index):
        return self.app.getTypeObject(index)

    def getObjectValue(self, index):
        return self.app.getObjectValue(index)

    def getPath(self, index):
        return self.app.getPath(index)

    def getReference(self, index):
        return self.app.getReference(index)

    def getString(self, index):
        return self.app.getString(index)

    def getType(self, index):
        return self.app.getType(index)

    def getUnitDoubleType(self, index):
        return self.app.getUnitDoubleType(index)

    def getUnitDoubleValue(self, index):
        return self.app.getUnitDoubleValue(index)

    def putBoolean(self, key, value):
        self.app.putBoolean(key, value)

    def putClass(self, value):
        self.app.putClass(value)

    def putDouble(self, key, value):
        return self.app.putDouble(key, value)

    def putEnumerated(self, DesiredClass, EnumType, Value):
        return self.app.putEnumerated(DesiredClass, EnumType, Value)

    def putReference(self, key, value):
        return self.app.putReference(key, value)

    def putObject(self, key, classID, value):
        return self.app.putObject(key, classID, value)



from photoshop import Session


def do_something(photoshop_api):
    print(photoshop_api.active_document)
    print("Do something.")


with Session(callback=do_something) as ps:
    ps.echo(ps.active_document.name)
    ps.alert(ps.active_document.name)



"""Add slate information dynamically。"""

from datetime import datetime
import os

from photoshop import Session

file_path = os.path.join(os.path.dirname(__file__), "slate_template.psd")

with Session(file_path, action="open", auto_close=True) as ps:
    layer_set = ps.active_document.layerSets.getByName("template")
    data = {
        "project name": "test_project",
        "datetime": datetime.today().strftime('%Y-%m-%d')
    }
    for layer in layer_set.layers:
        if layer.kind == "TextLayer":
            layer.textItem.contents = data[layer.textItem.contents.strip()]

    jpg_file = "d:/photoshop_slate.jpg"
    ps.active_document.saveAs(jpg_file, ps.JPEGSaveOptions())
    os.startfile(jpg_file)



from photoshop._core import Photoshop
from photoshop.colors.cmyk import CMYKColor
from photoshop.colors.hsb import HSBColor
from photoshop.colors.lab import LabColor
from photoshop.colors.rgb import RGBColor


class SolidColor(Photoshop):
    object_name = 'SolidColor'

    def __init__(self, parent=None):
        super().__init__(parent=parent)
        # Initialize color.
        self.rgb = RGBColor(self.app.rgb)
        self.cmyk = CMYKColor(self.app.cmyk)
        self.hsb = HSBColor(self.app.hsb)
        self.lab = LabColor(self.app.lab)

    @property
    def cmyk(self):
        """

        Returns:
            photoshop.colors.cmyk.CMYKColor:

        """
        return CMYKColor(self.app.cmyk)

    @cmyk.setter
    def cmyk(self, value):
        self.app.cmyk = value

    @property
    def gray(self):
        return self.app.gray

    @property
    def hsb(self):
        return HSBColor(self.app.hsb)

    @hsb.setter
    def hsb(self, value):
        self.app.hsb = value

    @property
    def lab(self):
        return LabColor(self.app.lab)

    @lab.setter
    def lab(self, value):
        self.app.lab = value

    @property
    def model(self):
        return self.app.model

    @property
    def nearestWebColor(self):
        return self.app.NearestWebColor

    @property
    def rgb(self):
        return RGBColor(self.app.rgb)

    @rgb.setter
    def rgb(self, value):
        self.app.rgb = value

    def isEqual(self, color):
        return self.app.isEqual(color)



""" This script demonstrates how you can use the action manager

to execute the Crystallize filter.
In order to find all the IDs, see https://helpx.adobe.com/photoshop/kb/downloadable-plugins-and-content.html#ScriptingListenerplugin
This blog here explains what a script listener is http://blogs.adobe.com/crawlspace/2006/05/installing_and_1.html

References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/ApplyCrystallizeFilterAction.py

"""


import os

from photoshop import Session

fileName = os.path.join(os.path.dirname(__file__), "layer_comps.psd")

with Session(fileName, "open") as ps:
    nLayerSets = ps.active_document.layerSets
    print(len(nLayerSets))
    nArtLayers = ps.active_document.layerSets.item(len(nLayerSets)).artLayers

    # get the last layer in LayerSets
    ps.active_document.activeLayer = ps.active_document.layerSets.item(
        len(nLayerSets)).artLayers.item(len(nArtLayers))


    def applyCrystallize(cellSize):
        cellSizeID = ps.app.CharIDToTypeID("ClSz")
        eventCrystallizeID = ps.app.CharIDToTypeID("Crst")

        filterDescriptor = ps.ActionDescriptor
        filterDescriptor.putInteger(cellSizeID, cellSize)

        ps.app.executeAction(eventCrystallizeID, filterDescriptor)


    applyCrystallize(25)



"""
References:
    https://github.com/lohriialo/photoshop-scripting-python/blob/master/ApplyFilters.py

"""
# selections in the open document.
import photoshop as ps
import os

# Start up Photoshop application
app = ps.Application()

# We don't want any Photoshop dialogs displayed during automated execution
app.displayDialogs = ps.DialogModes.DisplayNoDialogs

psPixels = 1
start_ruler_units = app.preferences.rulerUnits
if start_ruler_units is not psPixels:
    app.preferences.rulerUnits = psPixels

fileName = os.path.join(os.path.dirname(__file__), "layer_comps.psd")
docRef = app.open(fileName)
nLayerSets = len(list((i, x) for i, x in enumerate(docRef.layerSets))) - 1
nArtLayers = len(
    list((i, x) for i, x in enumerate(docRef.layerSets[nLayerSets].artLayers)),
)

active_layer = docRef.activeLayer = docRef.layerSets[nLayerSets].artLayers[
    nArtLayers
]
sel_area = ((0, 212), (300, 212), (300, 300), (0, 300))
docRef.selection.select(sel_area, ps.SelectionType.ReplaceSelection, 20, True)
print(active_layer.name)
active_layer.applyAddNoise(15, ps.NoiseDistribution.GaussianNoise, False)

backColor = ps.SolidColor()
backColor.hsb.hue = 0
backColor.hsb.saturation = 0
backColor.hsb.brightness = 100
app.backgroundColor = backColor

sel_area2 = ((120, 20), (210, 20), (210, 110), (120, 110))
docRef.selection.select(sel_area2, ps.SelectionType.ReplaceSelection, 25,
                        False)
active_layer.applyDiffuseGlow(9, 12, 15)
active_layer.applyGlassEffect(
    7, 3, 7, False, ps.TextureType.TinyLensTexture,
    None,
)
docRef.selection.deselect()

# Set ruler units back the way we found it.
if start_ruler_units is not psPixels:
    app.Preferences.RulerUnits = start_ruler_units



"""ActionDescriptor of photoshop scripting.

A record of key-value pairs for actions, such as those included on the
Adobe Photoshop Actions menu. The ActionDescriptor class is part of the Action
Manager functionality. For more details on the Action Manager,
see the Photoshop Scripting Guide.

"""
from photoshop._core import Photoshop


class ActionReference(Photoshop):
    object_name = 'ActionReference'

    def __init__(self):
        super().__init__()

    def putName(self, key, value):
        return self.app.putName(key, value)

    def putClass(self, value):
        return self.app.putClass(value)



"""Describe the distribution to distutils."""
from setuptools import find_packages
from setuptools import setup


with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='photoshop_python_api',
    author='Long Hao',
    package_dir={'': 'src'},
    url='https://github.com/loonghao/photoshop_python_api',
    version='0.7.2',
    packages=find_packages('./src'),
    setup_requires=['setuptools-scm', 'setuptools>=40.0'],
    author_email='hal.long@outllok.com',
    install_requires=['comtypes==1.1.7'],
    description=(
        'The API for using COM (Component Object Model) objects '
        'interfaces of Photoshop.'
    ),
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



from photoshop._core import Photoshop


class HSBColor(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)
        self.hue = 0
        self.saturation = 100
        self.brightness = 0

    @property
    def brightness(self):
        return self.app.brightness

    @brightness.setter
    def brightness(self, value):
        self.app.brightness = value

    @property
    def saturation(self):
        return self.app.saturation

    @saturation.setter
    def saturation(self, value):
        self.app.saturation = value

    @property
    def hue(self):
        """The hue value. Range: 0.0 to 360.0."""
        return self.app.hue

    @hue.setter
    def hue(self, value):
        self.app.hue = value



from photoshop._core import Photoshop
from photoshop.document import Document
from photoshop.enumerations import NewDocumentMode
from photoshop.enumerations import DocumentFill
from photoshop.enumerations import BitsPerChannelType


class Documents(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    def __len__(self):
        return self.length

    def add(self, width=960, height=540, resolution=72.0, name=None,
            mode=NewDocumentMode.NewRGB, initialFill=DocumentFill.White,
            pixelAspectRatio=None,
            bitsPerChannel=BitsPerChannelType.Document16Bits,
            colorProfileName=None
            ):
        """

        Args:
            width (int): The width of the document.
            height (int): The height of the document.
            resolution (int): The resolution of the document (in pixels per inch)
            name (str): The name of the document.
            mode (): The document mode.
            initialFill : The initial fill of the document.
            pixelAspectRatio: The initial pixel aspect ratio of the document.
            bitsPerChannel: The number of bits per channel.
            colorProfileName: The name of color profile for document.

        Returns:
            photoshop.Document: Document instance.

        """
        return Document(self.app.add(width, height, resolution,
                                     name, mode,
                                     initialFill, pixelAspectRatio,
                                     bitsPerChannel,
                                     colorProfileName))

    def __iter__(self):
        for layer in self.app:
            yield layer

    @property
    def length(self):
        return len(list(self.app))

    def getByName(self, document_name: str):
        return self.app.getByName(document_name)



from photoshop._core import Photoshop
from photoshop.layerSet import LayerSet
from photoshop.errors import PhotoshopPythonAPIError


class LayerSets(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    def __len__(self):
        return self.length

    def __getitem__(self, key):
        return LayerSet(self._layerSets[key])

    @property
    def _layerSets(self):
        return list(self.app)

    @property
    def length(self):
        return len(self._layerSets)

    def add(self):
        return LayerSet(self.app.add())

    def item(self, index):
        return LayerSet(self.app.item(index))

    def getByName(self, name):
        for layer in self.app:
            if name == layer.name:
                return LayerSet(layer)
        raise PhotoshopPythonAPIError(f'Could not find a LayerSet named '
                                      f'"{name}"')



# Import local modules
from photoshop._core import Photoshop


class CMYKColor(Photoshop):

    def __init__(self, parent):
        super().__init__(parent=parent)
        self.cyan = 75
        self.magenta = 68
        self.yellow = 67
        self.black = 90

    @property
    def black(self):
        """The black color value. Range: 0.0 to 100.0."""
        return self.app.black

    @black.setter
    def black(self, value):
        self.app.black = value

    @property
    def cyan(self):
        """The cyan color value. Range: 0.0 to 100.0."""
        return self.app.cyan

    @cyan.setter
    def cyan(self, value: int):
        self.app.cyan = value

    @property
    def magenta(self):
        """The magenta color value. Range: 0.0 to 100.0."""
        return self.app.magenta

    @magenta.setter
    def magenta(self, value: int):
        self.app.magenta = value

    @property
    def yellow(self):
        """The yellow color value. Range: 0.0 to 100.0."""
        return self.app.yellow

    @yellow.setter
    def yellow(self, value: float):
        self.app.yellow = value
